@@ -1,14 +1,14 @@
-import click
 import re
 import shutil
 import textwrap
 import typing as t
 
-from toot.entities import Account, Instance, Notification, Poll, Status, List
-from toot.utils import get_text, html_to_paragraphs
-from toot.wcstring import wc_wrap
+import click
 from wcwidth import wcswidth
 
+from toot.entities import Account, Instance, List, Notification, Poll, Status
+from toot.utils import get_text, html_to_paragraphs
+from toot.wcstring import wc_wrap
 
 DEFAULT_WIDTH = 80
 
@@ -21,6 +21,10 @@ def get_terminal_width() -> int:
     return shutil.get_terminal_size().columns
 
 
+def get_terminal_height() -> int:
+    return shutil.get_terminal_size().lines
+
+
 def get_width() -> int:
     return min(get_terminal_width(), get_max_width())
 
@@ -29,6 +33,16 @@ def print_warning(text: str):
     click.secho(f"Warning: {text}", fg="yellow", err=True)
 
 
+def get_continue():
+    click.secho(f"Press {green('Space')} or {green('Enter')} to continue, {yellow('Esc')} or {yellow('q')} to break.")
+    while True:
+        char = click.getchar()
+        if char == ' ' or char == '\r':
+            return True
+        if char == '\x1b' or char == 'q':
+            return False
+
+
 def print_instance(instance: Instance):
     width = get_width()
     click.echo(instance_to_text(instance, width))
@@ -99,7 +113,7 @@ def account_lines(account: Account, width: int) -> t.Generator[str, None, None]:
     if account.fields:
         for field in account.fields:
             name = field.name.title()
-            yield f'\n{yellow(name)}:'
+            yield f"\n{yellow(name)}:"
             yield from html_lines(field.value, width)
             if field.verified_at:
                 yield green("✓ Verified")
@@ -154,8 +168,9 @@ def print_list_accounts(accounts):
 
 
 def print_search_results(results):
-    accounts = results["accounts"]
-    hashtags = results["hashtags"]
+    accounts = results.get("accounts")
+    hashtags = results.get("hashtags")
+    statuses = results.get("statuses")
 
     if accounts:
         click.echo("\nAccounts:")
@@ -165,7 +180,12 @@ def print_search_results(results):
         click.echo("\nHashtags:")
         click.echo(", ".join([format_tag_name(tag) for tag in hashtags]))
 
-    if not accounts and not hashtags:
+    if statuses:
+        click.echo("\nStatuses:")
+        for status in statuses:
+            click.echo(f" * {green(status['id'])} {status['url']}")
+
+    if not accounts and not hashtags and not statuses:
         click.echo("Nothing found")
 
 
@@ -185,7 +205,7 @@ def status_lines(status: Status) -> t.Generator[str, None, None]:
     reblogged_by = status.account if status.reblog else None
     status = status.original
 
-    time = status.created_at.strftime('%Y-%m-%d %H:%M %Z')
+    time = status.created_at.strftime("%Y-%m-%d %H:%M %Z")
     username = "@" + status.account.acct
     spacing = width - wcswidth(username) - wcswidth(time) - 2
 
@@ -235,8 +255,11 @@ def html_lines(html: str, width: int) -> t.Generator[str, None, None]:
 
 def poll_lines(poll: Poll) -> t.Generator[str, None, None]:
     for idx, option in enumerate(poll.options):
-        perc = (round(100 * option.votes_count / poll.votes_count)
-            if poll.votes_count and option.votes_count is not None else 0)
+        perc = (
+            round(100 * option.votes_count / poll.votes_count)
+            if poll.votes_count and option.votes_count is not None
+            else 0
+        )
 
         if poll.voted and poll.own_votes and idx in poll.own_votes:
             voted_for = yellow(" ✓")
@@ -245,7 +268,7 @@ def poll_lines(poll: Poll) -> t.Generator[str, None, None]:
 
         yield f"{option.title} - {perc}% {voted_for}"
 
-    poll_footer = f'Poll · {poll.votes_count} votes'
+    poll_footer = f"Poll · {poll.votes_count} votes"
 
     if poll.expired:
         poll_footer += " · Closed"
@@ -272,9 +295,9 @@ def print_notification(notification: Notification):
         print_status(notification.status)
 
 
-def print_notifications(notifications: t.List[Notification]):
+def print_notifications(notifications: t.Iterable[Notification]):
     for notification in notifications:
-        if notification.type not in ['pleroma:emoji_reaction']:
+        if notification.type not in ["pleroma:emoji_reaction"]:
             print_divider()
             print_notification(notification)
     print_divider()
@@ -283,18 +306,30 @@ def print_notifications(notifications: t.List[Notification]):
 def print_notification_header(notification: Notification):
     account_name = format_account_name(notification.account)
 
-    if (notification.type == "follow"):
+    if notification.type == "follow":
         click.echo(f"{account_name} now follows you")
-    elif (notification.type == "mention"):
+    elif notification.type == "follow_request":
+        click.echo(f"{account_name} requested to follow you")
+    elif notification.type == "mention":
         click.echo(f"{account_name} mentioned you")
-    elif (notification.type == "reblog"):
-        click.echo(f"{account_name} reblogged your status")
-    elif (notification.type == "favourite"):
+    elif notification.type == "reblog":
+        click.echo(f"{account_name} boosted your status")
+    elif notification.type == "favourite":
         click.echo(f"{account_name} favourited your status")
-    elif (notification.type == "update"):
+    elif notification.type == "update":
         click.echo(f"{account_name} edited a post")
+    elif notification.type == "status":
+        click.echo(f"{account_name} posted a status")
+    elif notification.type == "poll":
+        click.echo("A poll you participated in has ended")
+    elif notification.type == "admin.sign_up":
+        click.echo(f"{account_name} has signed up")
+    elif notification.type == "admin.report":
+        click.echo(f"{account_name} filed a report")
     else:
-        click.secho(f"Unknown notification type: '{notification.type}'", err=True, fg="yellow")
+        click.secho(
+            f"Unknown notification type: '{notification.type}'", err=True, fg="yellow"
+        )
         click.secho("Please report an issue to toot.", err=True, fg="yellow")
 
 
@@ -316,6 +351,7 @@ def format_account_name(account: Account) -> str:
 
 # Shorthand functions for coloring output
 
+
 def blue(text: t.Any) -> str:
     return click.style(text, fg="blue")
 
