@@ -2,6 +2,7 @@ import logging
 import subprocess
 import urwid
 
+
 from concurrent.futures import ThreadPoolExecutor
 from typing import NamedTuple, Optional
 from datetime import datetime, timezone
@@ -9,17 +10,17 @@ from datetime import datetime, timezone
 from toot import api, config, __version__, settings
 from toot import App, User
 from toot.cli import get_default_visibility
-from toot.exceptions import ApiError
 from toot.utils.datetime import parse_datetime
 
 from .compose import StatusComposer
 from .constants import PALETTE
 from .entities import Status
+from .images import TuiScreen, load_image
 from .overlays import ExceptionStackTrace, GotoMenu, Help, StatusSource, StatusLinks, StatusZoom
 from .overlays import StatusDeleteConfirmation, Account
 from .poll import Poll
 from .timeline import Timeline
-from .utils import get_max_toot_chars, parse_content_links, copy_to_clipboard
+from .utils import get_max_toot_chars, parse_content_links, copy_to_clipboard, LRUCache
 from .widgets import ModalBox, RoundedLineBox
 
 logger = logging.getLogger(__name__)
@@ -35,7 +36,10 @@ class TuiOptions(NamedTuple):
     media_viewer: Optional[str]
     always_show_sensitive: bool
     relative_datetimes: bool
+    cache_size: int
     default_visibility: Optional[str]
+    image_format: Optional[str]
+    show_display_names: bool
 
 
 class Header(urwid.WidgetWrap):
@@ -93,12 +97,12 @@ class TUI(urwid.Frame):
     screen: urwid.BaseScreen
 
     @staticmethod
-    def create(app: App, user: User, args: TuiOptions):
+    def create(app: App, user: User, options: TuiOptions):
         """Factory method, sets up TUI and an event loop."""
-        screen = urwid.raw_display.Screen()
-        screen.set_terminal_properties(args.colors)
+        screen = TuiScreen()
+        screen.set_terminal_properties(options.colors)
 
-        tui = TUI(app, user, screen, args)
+        tui = TUI(app, user, screen, options)
 
         palette = PALETTE.copy()
         overrides = settings.get_setting("tui.palette", dict, {})
@@ -144,14 +148,19 @@ class TUI(urwid.Frame):
         self.followed_accounts = []
         self.preferences = {}
 
+        if self.options.cache_size:
+            self.cache_max = 1024 * 1024 * self.options.cache_size
+        else:
+            self.cache_max = 1024 * 1024 * 10  # default 10MB
+
         super().__init__(self.body, header=self.header, footer=self.footer)
 
     def run(self):
+        self.loop.set_alarm_in(0, lambda *args: self.async_load_account())
         self.loop.set_alarm_in(0, lambda *args: self.async_load_instance())
         self.loop.set_alarm_in(0, lambda *args: self.async_load_preferences())
         self.loop.set_alarm_in(0, lambda *args: self.async_load_timeline(
             is_initial=True, timeline_name="home"))
-        self.loop.set_alarm_in(0, lambda *args: self.async_load_followed_accounts())
         self.loop.run()
         self.executor.shutdown(wait=False)
 
@@ -327,8 +336,10 @@ class TUI(urwid.Frame):
                 # get the major version number of the server
                 # this works for Mastodon and Pleroma version strings
                 # Mastodon versions < 4 do not have translation service
+                # If the version is missing, assume 0 as a fallback
                 # Revisit this logic if Pleroma implements translation
-                ch = instance["version"][0]
+                version = instance["version"]
+                ch = "0" if not version else version[0]
                 self.can_translate = int(ch) > 3 if ch.isnumeric() else False
 
         return self.run_in_thread(_load_instance, done_callback=_done)
@@ -346,13 +357,23 @@ class TUI(urwid.Frame):
 
         return self.run_in_thread(_load_preferences, done_callback=_done)
 
+    def async_load_account(self):
+        def _load_account():
+            return api.verify_credentials(self.app, self.user).json()
+
+        def _done_account(account):
+            self.account = account
+            self.loop.set_alarm_in(0, lambda *_: self.async_load_followed_accounts())
+
+        self.run_in_thread(_load_account, done_callback=_done_account)
+
     def async_load_followed_accounts(self):
         def _load_accounts():
             try:
-                acct = f'@{self.user.username}@{self.user.instance}'
-                self.account = api.find_account(self.app, self.user, acct)
-                return api.following(self.app, self.user, self.account["id"])
-            except ApiError:
+                if self.account:
+                    return api.following(self.app, self.user, self.account["id"])
+                return []
+            except Exception:
                 # not supported by all Mastodon servers so fail silently if necessary
                 return []
 
@@ -538,11 +559,12 @@ class TUI(urwid.Frame):
         promise.add_done_callback(lambda *args: self.close_overlay())
 
     def goto_personal_timeline(self):
-        account_name = f"{self.user.username}@{self.user.instance}"
-
-        self.timeline_generator = api.account_timeline_generator(
-            self.app, self.user, account_name, reblogs=True, limit=40)
-        promise = self.async_load_timeline(is_initial=True, timeline_name=f"personal {account_name}")
+        if not self.account:
+            return
+        self.timeline_generator = api.account_timeline_generator_by_id(
+            self.app, self.user, self.account["id"], reblogs=True, limit=40)
+        promise = self.async_load_timeline(
+            is_initial=True, timeline_name=f"personal {self.user.username}")
         promise.add_done_callback(lambda *args: self.close_overlay())
 
     def goto_list_timeline(self, list_item):
@@ -646,7 +668,7 @@ class TUI(urwid.Frame):
         account = api.whois(self.app, self.user, account_id)
         relationship = api.get_relationship(self.app, self.user, account_id)
         self.open_overlay(
-            widget=Account(self.app, self.user, account, relationship),
+            widget=Account(self.app, self.user, account, relationship, self.options),
             title="Account",
         )
 
@@ -755,6 +777,27 @@ class TUI(urwid.Frame):
 
         return self.run_in_thread(_delete, done_callback=_done)
 
+    def async_load_image(self, timeline, status, path, placeholder_index):
+        def _load():
+            # don't bother loading images for statuses we are not viewing now
+            if timeline.get_focused_status().id != status.id:
+                return
+
+            if not hasattr(timeline, "images"):
+                timeline.images = LRUCache(cache_max_bytes=self.cache_max)
+
+            img = load_image(path)
+            if img:
+                timeline.images[str(hash(path))] = img
+
+        def _done(loop):
+            # don't bother loading images for statuses we are not viewing now
+            if timeline.get_focused_status().id != status.id:
+                return
+            timeline.update_status_image(status, path, placeholder_index)
+
+        return self.run_in_thread(_load, done_callback=_done)
+
     def copy_status(self, status):
         # TODO: copy a better version of status content
         # including URLs
